#!/usr/bin/python

#############################################################################
##
## Copyright (C) 2018 The Qt Company Ltd.
## Contact: https://www.qt.io/licensing/
##
## This file is part of the test suite of Qt for Python.
##
## $QT_BEGIN_LICENSE:GPL-EXCEPT$
## Commercial License Usage
## Licensees holding valid commercial Qt licenses may use this file in
## accordance with the commercial license agreement provided with the
## Software or, alternatively, in accordance with the terms contained in
## a written agreement between you and The Qt Company. For licensing terms
## and conditions see https://www.qt.io/terms-conditions. For further
## information use the contact form at https://www.qt.io/contact-us.
##
## GNU General Public License Usage
## Alternatively, this file may be used under the terms of the GNU
## General Public License version 3 as published by the Free Software
## Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
## included in the packaging of this file. Please review the following
## information to ensure the GNU General Public License requirements will
## be met: https://www.gnu.org/licenses/gpl-3.0.html.
##
## $QT_END_LICENSE$
##
#############################################################################

'''Test cases for QSensor'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtSensors import QSensor, QSensorReading


class QSensorTest(unittest.TestCase):
    def test(self):
        for sensorType in QSensor.sensorTypes():
            identifiers = QSensor.sensorsForType(sensorType)
            values = []
            error = ''
            for identifier in identifiers:
                sensor = QSensor(sensorType, None)
                sensor.setIdentifier(identifier)
                if sensor.connectToBackend():
                    usedIdentifier = identifier
                    reading = sensor.reading()
                    if reading:
                        for i in range(0, reading.valueCount()):
                            values.append(reading.value(i))
                        break
                    else:
                        error = "Unable to obtain reading"
                else:
                    error = "Unable to connect to backend"
            if values:
                print('Sensor ', sensorType, usedIdentifier, values)
            else:
                print(f"{sensorType}: {error}", file=sys.stderr)


if __name__ == '__main__':
    unittest.main()
