#############################################################################
##
## Copyright (C) 2021 The Qt Company Ltd.
## Contact: https://www.qt.io/licensing/
##
## This file is part of the test suite of Qt for Python.
##
## $QT_BEGIN_LICENSE:GPL-EXCEPT$
## Commercial License Usage
## Licensees holding valid commercial Qt licenses may use this file in
## accordance with the commercial license agreement provided with the
## Software or, alternatively, in accordance with the terms contained in
## a written agreement between you and The Qt Company. For licensing terms
## and conditions see https://www.qt.io/terms-conditions. For further
## information use the contact form at https://www.qt.io/contact-us.
##
## GNU General Public License Usage
## Alternatively, this file may be used under the terms of the GNU
## General Public License version 3 as published by the Free Software
## Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
## included in the packaging of this file. Please review the following
## information to ensure the GNU General Public License requirements will
## be met: https://www.gnu.org/licenses/gpl-3.0.html.
##
## $QT_END_LICENSE$
##
#############################################################################

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication

from PySide6.QtCore import QFile, QObject, QTimer, Signal, Slot
from PySide6.QtWidgets import QSlider


class C(QObject):
    custom_signal = Signal()


class D(C):
    pass


class TestSignalInstance(unittest.TestCase):
    def test_signal_instances_are_equal(self):
        o = QTimer()
        self.assertTrue(o.timeout == o.timeout)

    def test_inherited_signal_instances_are_equal(self):
        o = QFile()
        self.assertTrue(o.readyRead == o.readyRead)

    def test_custom_signal_instances_are_equal(self):
        o = C()
        self.assertTrue(o.custom_signal == o.custom_signal)

    def test_custom_inherited_signal_instances_are_equal(self):
        o = D()
        self.assertTrue(o.custom_signal == o.custom_signal)


class MyWidget(QSlider):
    valueChanged = Signal(tuple)

    def __init__(self):
        super().__init__()
        self.valueChanged.connect(self._on_change)

    def setValue(self, value):
        self.valueChanged.emit(value)

    @Slot()
    def _on_change(self, new_value):
        print("new_value:", new_value)
        global result
        result = new_value


class TestRightOrder(UsesQApplication):
    def test_rightOrder(self):
        wdg = MyWidget()

        # PYSIDE-1751: Fixes the wrong behavior we got on >=6.2
        #     PySide <=6.1.3 prints "new_value: (30, 40)"
        #     PySide >=6.2 prints "new_value: 0"
        wdg.setValue((30, 40))
        self.assertEqual(result, (30, 40))


if __name__ == '__main__':
    unittest.main()
